#!/usr/bin/wish

#****h* GalaxyTools/shipyard
# NAME
#   shipyard -- a small tool to design ships.
# FUNCTION
#   This tools lets you design ships interactively. You can tweak
#   all 5 design parameters and the four technology levels. The 
#   program then computes the mass, speed, attack, and defenses of the
#   ships. This for a ship with an empty cargo bay and a fully loaded
#   cargo bay.
# BUGS
#   Go to fslothouber@acm.org
#*****

wm title . "Shipyard -- ship design tool"

frame .setting -relief groove -borderwidth 3
frame .stats   -relief groove -borderwidth 3
pack  .setting -side bottom -fill x 
pack  .stats   -side top    -fill x 


# Create the Menus

menu  .menu -tearoff 0
set   base .menu.file
menu $base -tearoff 0
.menu add cascade -label File -menu $base
$base add command -label "Exit" -command {exit}

set   base .menu.help
.menu add cascade -label Help -menu $base

menu $base -tearoff 0
$base add command -label "Manual" -command {AboutSY 1}
$base add command -label "About"  -command {AboutSY 0}
. configure -menu .menu

# Create settings frame with the slidebars (scales)

set base .setting
scale $base.dt -label "Drive Technolgy" -from 1.0 -to 10.0 -resolution 0.01 \
      -orient horizontal -length 8c -variable drivet -command UpdateStats
scale $base.wt -label "Weapons Technolgy" -from 1.0 -to 10.0 -resolution 0.01 \
      -orient horizontal -length 8c -variable weaponst -command UpdateStats
scale $base.st -label "Shields Technolgy" -from 1.0 -to 10.0 -resolution 0.01 \
       -orient horizontal -length 8c -variable shieldst -command UpdateStats
scale $base.ct -label "Cargo Technolgy" -from 1.0 -to 10.0 -resolution 0.01 \
       -orient horizontal -length 8c -variable cargot -command UpdateStats

scale $base.drive -label Drive -from 0 -to 200.0 -resolution 0.1 \
      -orient horizontal -length 8c -variable drive -command UpdateStats
scale $base.attacks -label Attacks -from 0 -to 500 -resolution 1 \
      -orient horizontal -length 8c -variable attacks -command UpdateStats
scale $base.weapons -label Weapon -from 0 -to 200.0 -resolution 0.1 \
      -orient horizontal -length 8c -variable weapons -command UpdateStats
scale $base.shields -label Shields -from 0 -to 200.0 -resolution 0.1 \
      -orient horizontal -length 8c -variable shields -command UpdateStats
scale $base.cargo -label Cargo -from 0 -to 200.0 -resolution 0.1 \
      -orient horizontal -length 8c -variable cargo -command UpdateStats


set row 0
grid $base.drive   -column 0 -row $row -sticky ew -ipadx 1   
grid $base.dt      -column 1 -row $row -sticky ew -ipadx 1  
incr row
grid $base.attacks -column 0 -row $row -sticky ew -ipadx 1  
incr row
grid $base.weapons -column 0 -row $row -sticky ew -ipadx 1  
grid $base.wt      -column 1 -row $row -sticky ew -ipadx 1  
incr row
grid $base.shields -column 0 -row $row -sticky ew -ipadx 1  
grid $base.st      -column 1 -row $row -sticky ew -ipadx 1  
incr row
grid $base.cargo   -column 0 -row $row -sticky ew -ipadx 1  
grid $base.ct      -column 1 -row $row -sticky ew -ipadx 1  

# Create statistics frame

set base .stats
label $base.lpar       -text "Cargo Bay"                -fg navy -anchor w
label $base.lemp       -text "Empty"                    -fg red -anchor  e
label $base.lfull      -text "Full"                     -fg red -anchor  e
label $base.lmass      -text "Mass:"                    -fg navy -anchor w
label $base.lload      -text "Load:"                    -fg navy -anchor w
label $base.lspeed     -text "Speed:"                   -fg navy -anchor w
label $base.leffgun    -text "Eff. gun:"                 -fg navy -anchor w
label $base.lshld      -text "Immune to an eff. gun of:" -fg navy -anchor w
label $base.massemp    -textvariable massemp     -anchor e 
label $base.massfull   -textvariable massfull    -anchor e 
label $base.load       -textvariable load        -anchor e 
label $base.effgun     -textvariable effgun      -anchor e 
label $base.effgun2    -textvariable effgun      -anchor e 
label $base.speedemp   -textvariable speedemp    -anchor e 
label $base.speedfull  -textvariable speedfull   -anchor e 
label $base.shldemp    -textvariable shldemp     -anchor e 
label $base.shldfull   -textvariable shldfull    -anchor e 

set row 0
grid $base.lpar     -column 0 -row $row -sticky e 
grid $base.lemp     -column 1 -row $row -sticky e 
grid $base.lfull    -column 2 -row $row -sticky e 
incr row
grid $base.lmass    -column 0 -row $row -sticky e 
grid $base.massemp  -column 1 -row $row -sticky we -ipadx 1  
grid $base.massfull -column 2 -row $row -sticky we -ipadx 1  
incr row
grid $base.lload    -column 0 -row $row -sticky e
grid $base.load     -column 2 -row $row -sticky we -ipadx 1  
incr row
grid $base.lspeed    -column 0 -row $row -sticky e
grid $base.speedemp  -column 1 -row $row -sticky we -ipadx 1  
grid $base.speedfull -column 2 -row $row -sticky we -ipadx 1  
incr row
grid $base.leffgun   -column 0 -row $row -sticky e
grid $base.effgun    -column 1 -row $row -sticky we -ipadx 1  
grid $base.effgun2   -column 2 -row $row -sticky we -ipadx 1  
incr row
grid $base.lshld     -column 0 -row $row -sticky e 
grid $base.shldemp   -column 1 -row $row -sticky we -ipadx 1  
grid $base.shldfull  -column 2 -row $row -sticky we -ipadx 1  
incr row


#****** ShipYard/AboutSY
# NAME
#   AboutSY -- pop-up window with manual or about message.
# INPUTS
#   kind --  ==  0 pop-up about message
#            !=  0 pop-up the manual
#******
#

proc AboutSY {kind} {
    set base .about
    toplevel $base

    label $base.msg -wraplength 6i -justify left 
    if {$kind == 0} { 
	wm title $base "About"
	$base.msg configure -text "Shipyard Version 2.0, fslothouber@acm.org"
    } else {
	wm title $base "Help"
	$base.msg configure -text "
This tool lets you design ships interactively. You can tweak
all 5 design parameters and the four technology levels. The 
program then computes the mass, speed, effective gun, and 
defenses of the	ships. This for a ship with an empty cargo bay 
and a fully loaded cargo bay. If you click to the left or right 
of the slidebar ruler, you can fine-tune the parameters in 
steps of 0.01 or 0.1."
    }
    button $base.dismiss -text Dismiss -command "destroy .about"
    pack $base.msg -side top 
    pack $base.dismiss -side bottom 
}


#****** ShipYard/UpdateStats
# NAME
#   UpdateStats 
# FUNCTION
#   This is called each time the user tweaks one of the parameters.
#   All variables are recomputed based on the new settings.
#******
#

proc UpdateStats {x} {
    global drive attacks weapons shields cargo
    global drivet weaponst cargot shieldst
    global massemp massfull load shldemp shldfull speedemp speedfull
    global effgun

    set mass [shipmass]
    set massemp  [format "%.2f" $mass]
    if {$mass>0} {
	set load        [format "%.3f" [expr \
		$cargot*($cargo+($cargo*$cargo/10.0))]]
	set speedemp    [format "%.3f" [expr 20*$drivet*$drive/$mass]]
	set effgun      [format "%.3f" [expr $weapons*$weaponst]]
	set shldemp     [format "%.3f" [expr \
		$shieldst*$shields/pow(($mass),0.33333)*0.776808]]
	if {$load == 0} {
	    set speedfull "-"
	    set shldfull  "-"
	    set massfull  "-"
	} else {
	    set massfull [format "%.2f" [expr $mass + $load/$cargot]]
	    set speedfull   [format "%.3f" \
		    [expr 20*$drivet*$drive/($mass+$load)]]
	    set shldfull    [format "%.3f" [expr \
		    $shieldst*$shields/pow(($mass+$load),0.33333)*0.776808]]
	}
    }
}

#****** ShipYard/shipmass
# NAME
#   shipmass -- compute mass of ship and do some sanity checks
#*****

proc shipmass {} {
    global drive attacks weapons shields cargo
    if {[expr ($drive < 1) && ($drive > 0)]} {
	set drive 1
    }
    if {[expr ($weapons < 1) && ($weapons > 0)]} {
	set weapons 1
	if {$attacks == 0} {
	    set attacks 1
	}
    }
    if {[expr ($shields < 1) && ($shields > 0)]} {
	set shields 1
    }
    if {[expr ($cargo < 1) && ($cargo > 0)]} {
	set cargo 1
    }
    set mass [expr $drive+$shields+$cargo]
    if {$attacks == 1} {
	set mass [expr $mass+$weapons]
    } elseif {$attacks > 1} {
 	set mass [expr $mass+$weapons+($attacks-1)*$weapons/2.0]
    }
    return $mass
}



